/******************************************************************************
 * The copyright in this software is being made available under the BSD        *
 * License, included below. This software may be subject to other third party  *
 * and contributor rights, including patent rights, and no such rights are     *
 * granted under this license.                                                 *
 *    <OWNER> = ISO                                                            *
 *    <ORGANIZATION> = ISO                                                     *
 *    <YEAR> = 2015                                                            *
 *    Copyright (c) 2012-2015, ISO                                             *
 *                                                                             *
 * All rights reserved.                                                        *
 * Redistribution and use in source and binary forms, with or without          *
 * modification, are permitted provided that the following conditions are met: *
 *       * Redistributions of source code must retain the above copyright      *
 *         notice,this list of conditions and the following disclaimer.        *
 *       * Redistributions in binary form must reproduce the above             *
 *         copyright notice, this list of conditions and the following         *
 *         disclaimer in the documentation and/or other materials provided     *
 *         with the distribution.                                              *
 *       * Neither the name of the <ORGANIZATION> nor the names of its         *
 *         contributors may be used to endorse or promote products derived     *
 *         from this software without specific prior written permission.       *
 *                                                                             *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" *
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE  *
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE   *
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR         *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF        *
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS    *
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN     *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)     *
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE  *
 * POSSIBILITY OF SUCH DAMAGE.                                                 *
 ******************************************************************************/

#ifndef __PARAMETERS_PARSER_H__
#define __PARAMETERS_PARSER_H__

#include <string>
#include "jh_encoder.h"
#include "jh_decoder.h"
#include "tonemap.h"

/*------------------------------------------------------------------------------ */
/* Processing parameter struct parser routines                                   */

/* NOTE: marker fields of the encoder and decoder param structs are ignored by
 *       the routines below and must be initialized externally. 
 */

/* Populate encoder parameters from JSON-formatted parameters file.  Returns 0 on success,
 *    non-zero on failure.  Defaults are used in case of missing or malformed values. 
 */
extern int GetEncoderParameters(jh_encoder_params_t* encoderParams, const char *paramsfile, jh_error_t *err);

/* Populate decoder parameters from JSON-formatted parameters file.  Returns 0 on success,
 *    non-zero on failure.  Defaults are used in case of missing or malformed values. 
 */
extern int GetDecoderParameters(jh_decoder_params_t* decoderParams, const char *paramsfile, jh_error_t *err);

/* Populate tonemapping parameters from JSON-formatted parameters file.  Returns 0 on success,
 *    non-zero on failure.  Defaults are used in case of missing or malformed values. 
 */
extern int GetTonemappingParameters(tmo_params_t* tmoParams, const char *paramsfile, jh_error_t *err);

/* Allocate and populate a binary data blob with ICC profile data and it's size, using the
 *    path specified in the params file.  Returns 0 on success, non-zero on failure.  No default
 *    values are substituted in the case of missing or malformed values. 
 */
extern int GetICCProfile(unsigned char **icc_profile_data, size_t *icc_profile_size, const char *paramsfile, jh_error_t *err);

/*------------------------------------------------------------------------------ */
/* Generic parser routines                                                       */

extern int GetIntParameterWithDefault(const char *paramsfile, const char *key, const int defaultValue, jh_error_t *err);

extern const std::string GetStringParameterWithDefault(const char *paramsfile, const char *key, const char *defaultValue, jh_error_t *err);

/* Add others here as needed... */


/*------------------------------------------------------------------------------ */
/* Defaults routines                                                             */

/* Fill decoder parameters with default values, for cases when no parameters file is given. */
extern void GetDefaultDecoderParameters(jh_decoder_params_t* decoderParams, jh_error_t *err);

/* Fill tonemapping operator parameters with default values, for cases when no parameters file is given. */
extern void GetDefaultTonemappingParameters(tmo_params_t* tmoParams, jh_error_t *err);

#endif  /* __PARAMETERS_PARSER_H__ */
